
<?php

require_once dirname(__FILE__) . '/../../includes/autoload.php';

class KredivoNotificationModuleFrontController extends ModuleFrontController
{
    /**
     * @see FrontController::postProcess()
     */
    public function postProcess()
    {
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {

            Kredivo_Api::close_connection(Kredivo_Api::response());

            $input = new Kredivo_Input();
            Logger::addLog("Kredivo : " . $input->get_json(), 1);
            Kredivo_Log::debug($input->get_json());

            // Setup Config
            Kredivo_Config::$is_production = $this->module->isProduction();

            try {
                $confirmation = Kredivo_Api::confirm_order(array(
                    'transaction_id' => $input->transaction_id,
                    'signature_key'  => $input->signature_key,
                ));

                Logger::addLog("Kredivo : " . json_encode($confirmation), 1);
                Kredivo_Log::debug($confirmation);

                if (strtolower($confirmation->status) == 'ok') {
                    $this->_validConfirmation($confirmation);
                }
            } catch (Exception $e) {
                Kredivo_Log::debug($e->getMessage());
            }

            exit();
        }
    }

    private function _validConfirmation($confirmation)
    {
        $order             = new Order((int) $confirmation->order_id);
        $history           = new OrderHistory();
        $history->id_order = (int) $confirmation->order_id;

        $fraud_status = strtolower($confirmation->fraud_status);
        $order_status = Configuration::get('PS_OS_ERROR');

        if ($fraud_status == 'accept') {
            $transaction_status = strtolower($confirmation->transaction_status);
            switch ($transaction_status) {
                case 'settlement':
                    $order_status = Configuration::get('PS_OS_PAYMENT');
                    break;
                case 'pending':
                    $order_status = Configuration::get('PS_OS_PREPARATION');
                    break;
                case 'cancel':
                    $order_status = Configuration::get('PS_OS_CANCELED');
                    break;
                default:
                    $order_status;
            }
        }

        $history->changeIdOrderState((int) $order_status, $order);
        $history->addWithemail(true);
    }
}
