<?php
/**
 * @Author: Dodik Gaghan
 * @Date:   2016-06-25 11:18:55
 * @Last Modified by:   Dodik Gaghan
 * @Last Modified time: 2016-08-23 20:09:17
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class Kredivo extends PaymentModule
{
    private $_html       = '';
    private $_postErrors = array();

    private $kredivo_environment;
    private $kredivo_key;
    private $kredivo_title;
    private $kredivo_desc;

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->name            = 'kredivo';
        $this->version         = '2.1';
        $this->author          = 'FinAccel';
        $this->className       = 'kredivo';
        $this->currencies      = true;
        $this->currencies_mode = 'checkbox';
        $this->tab             = 'payments_gateways';
        $this->bootstrap       = true;

        $this->_loadConfiguration();
        parent::__construct();

        // Information
        $this->page             = basename(__FILE__, '.php');
        $this->displayName      = $this->l('Kredivo');
        $this->description      = $this->l("Kredivo is South East Asia's first real time credit financing solution that gives your customers the ability to buy on credit in real time without the credit card.");
        $this->confirmUninstall = $this->l('Are you sure about uninstalling Kredivo ?');

        $this->_loadContext();
    }

    public function install()
    {
        if (!parent::install() || !$this->registerHook('payment')) {
            return false;
        }
        return true;
    }

    public function uninstall()
    {
        if (!Configuration::deleteByName('PS_KREDIVO_ENVIRONMENT')
            || !Configuration::deleteByName('PS_KREDIVO_TITLE')
            || !Configuration::deleteByName('PS_KREDIVO_DESC')
            || !Configuration::deleteByName('PS_KREDIVO_SERVER_KEY')
            || !parent::uninstall()) {
            return false;
        }

        return true;
    }

    public function getContent()
    {
        if (Tools::isSubmit('btnSubmit')) {
            $this->_postValidation();
            if (!count($this->_postErrors)) {
                $this->_postProcess();
            } else {
                foreach ($this->_postErrors as $err) {
                    $this->_html .= '<div class="alert alert-danger error">' . $err . '</div>';
                }
            }
        } else {
            $this->_html .= '<br />';
        }

        $this->_html .= $this->context->smarty->fetch(__DIR__ . '/views/templates/hook/info.tpl');

        if ($this->_isOldPrestashop()) {
            $this->_displayFormOld();
        } else {
            $this->_displayFormNew();
        }

        return $this->_html;
    }

    public function hookPayment($params)
    {

        if (!$this->active || !$this->checkCurrency($params['cart'])) {
            return;
        }

        $this->smarty->assign(array(
            'this_path'     => $this->_path,
            'this_path_ssl' => Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' . $this->name . '/',
        ));

        return $this->display(__FILE__, 'payment.tpl');
    }

    public function checkCurrency($cart)
    {
        $currency_order    = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }

        return false;
    }

    public function getBaseUri()
    {
        return Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__;
    }

    public function getNotificationUri()
    {
        return $this->context->link->getModuleLink('kredivo', 'notification');
    }

    public function getOrderStatusUri()
    {
        return $this->context->link->getModuleLink('kredivo', 'status');
    }

    public function isProduction()
    {
        return (Configuration::get('PS_KREDIVO_ENVIRONMENT') == 'production') ? true : false;
    }

    public function getLanguageID()
    {
        return $this->context->language->id;
    }

    // ================== PRIVATE AREA

    private function _postValidation()
    {
        if (Tools::isSubmit('btnSubmit')) {
            if (!Tools::getValue('environment')) {
                $this->_postErrors[] = $this->l('Environment is required.');
            } elseif (!Tools::getValue('server_key')) {
                $this->_postErrors[] = $this->l('Server key is required.');
            }

        }
    }

    private function _postProcess()
    {
        if (Tools::isSubmit('btnSubmit')) {
            Configuration::updateValue('PS_KREDIVO_ENVIRONMENT', Tools::getValue('environment'));
            Configuration::updateValue('PS_KREDIVO_SERVER_KEY', Tools::getValue('server_key'));
            Configuration::updateValue('PS_KREDIVO_TITLE', Tools::getValue('title'));
            Configuration::updateValue('PS_KREDIVO_DESC', Tools::getValue('description'));
        }
        $this->_html .= '<div class="alert alert-success conf confirm">' . $this->l('Settings updated') . '</div>';
    }

    // Display Form for New Version || Prestashop > 1.5
    private function _displayFormNew()
    {
        $environments = array(
            array(
                'id_option' => 'sandbox',
                'name'      => 'Sandbox',
            ),
            array(
                'id_option' => 'production',
                'name'      => 'Production',
            ),
        );
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => 'Basic Information',
                    'icon'  => 'icon-cogs',
                ),
                'input'  => array(
                    array(
                        'type'     => 'select',
                        'label'    => 'Environment',
                        'name'     => 'environment',
                        'required' => true,
                        'options'  => array(
                            'query' => $environments,
                            'id'    => 'id_option',
                            'name'  => 'name',
                        ),
                        'class'    => 'v2_settings sensitive',
                        'desc'     => 'Place kredivo in test mode using sandbox api endpoint.',
                    ),
                    array(
                        'type'  => 'text',
                        'label' => 'Title',
                        'name'  => 'title',
                        'desc'  => 'This controls the title which the user sees during checkout.',
                        'class' => 'v1_vtdirect_settings v2_settings',
                    ),
                    array(
                        'type'  => 'text',
                        'label' => 'Description',
                        'name'  => 'description',
                        'desc'  => 'This controls the description which the user sees during checkout.',
                        'class' => 'v1_vtdirect_settings v2_settings',
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => 'Server Key',
                        'name'     => 'server_key',
                        'required' => true,
                        'desc'     => 'This server_key is generated by Kredivo.',
                        'class'    => 'v1_vtdirect_settings v2_settings sensitive',
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );

        $helper                           = new HelperForm();
        $lang                             = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper->show_toolbar             = false;
        $helper->table                    = $this->table;
        $helper->default_form_language    = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form                = array();
        $helper->id                       = (int) Tools::getValue('id_carrier');
        $helper->identifier               = $this->identifier;
        $helper->submit_action            = 'btnSubmit';
        $helper->currentIndex             = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;

        $helper->token    = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => array(
                'environment' => Tools::getValue('environment', Configuration::get('PS_KREDIVO_ENVIRONMENT')),
                'title'       => Tools::getValue('title', Configuration::get('PS_KREDIVO_TITLE')),
                'description' => Tools::getValue('description', Configuration::get('PS_KREDIVO_DESC')),
                'server_key'  => Tools::getValue('server_key', Configuration::get('PS_KREDIVO_SERVER_KEY')),
            ),
            'languages'    => $this->context->controller->getLanguages(),
            'id_language'  => $this->getLanguageID(),
        );

        $this->_html .= $helper->generateForm(array($fields_form));
    }

    // Display Form for Old Version
    private function _displayFormOld()
    {
        $this->context->smarty->assign(
            array(
                'form_url'     => Tools::htmlentitiesUTF8($_SERVER['REQUEST_URI']),
                'environments' => array('sandbox' => 'Sandbox', 'production' => 'Production'),
                'environment'  => htmlentities(Configuration::get('PS_KREDIVO_ENVIRONMENT'), ENT_COMPAT, 'UTF-8'),
                'title'        => htmlentities(Configuration::get('PS_KREDIVO_TITLE'), ENT_COMPAT, 'UTF-8'),
                'description'  => htmlentities(Configuration::get('PS_KREDIVO_DESC'), ENT_COMPAT, 'UTF-8'),
                'server_key'   => htmlentities(Configuration::get('PS_KREDIVO_SERVER_KEY'), ENT_COMPAT, 'UTF-8'),
            )
        );

        $this->_html .= $this->context->smarty->fetch(__DIR__ . '/views/templates/hook/admin.tpl');
    }

    private function _loadConfiguration()
    {
        $config = Configuration::getMultiple(array(
            'PS_KREDIVO_ENVIRONMENT',
            'PS_KREDIVO_TITLE',
            'PS_KREDIVO_DESC',
            'PS_KREDIVO_SERVER_KEY',
        ));

        if (isset($config['PS_KREDIVO_ENVIRONMENT'])) {
            $this->kredivo_environment = $config['PS_KREDIVO_ENVIRONMENT'];
        }
        if (isset($config['PS_KREDIVO_SERVER_KEY'])) {
            $this->kredivo_key = $config['PS_KREDIVO_SERVER_KEY'];
        }
        if (isset($config['PS_KREDIVO_TITLE'])) {
            $this->kredivo_title = $config['PS_KREDIVO_TITLE'];
        }
        if (isset($config['PS_KREDIVO_DESC'])) {
            $this->kredivo_desc = $config['PS_KREDIVO_DESC'];
        }

        if (!isset($this->kredivo_environment) || !isset($this->kredivo_key)) {
            $this->warning = $this->l('Environment and server key must be configured in order to use this module correctly.');
        }

        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency set for this module');
        }

    }

    private function _loadContext()
    {
        if (class_exists('Context')) {
            $this->context = Context::getContext();
        } else {
            global $smarty, $cookie;
            $this->context         = new StdClass();
            $this->context->smarty = $smarty;
            $this->context->cookie = $cookie;

            if (array_key_exists('cart', $GLOBALS)) {
                global $cart;
                $this->context->cart = $cart;
            }

            if (array_key_exists('link', $GLOBALS)) {
                global $link;
                $this->context->link = $link;
            }
        }
    }

    private function _isOldPrestashop()
    {
        // Check if prestashop version lower than 1.5
        return version_compare(Configuration::get('PS_VERSION_DB'), '1.5') == -1;
    }
}
